// # 
// # Editor     : Lauren from DFRobot
// # Date       : 22.02.2013
// # E-Mail	: Lauren.pan@dfrobot.com
// # Version    : 0.02

// # Project name: iOS to Xboard

// # Description:

// # Connection:
// #   DHT11 temperature and humidity sensor connected to D2 of the xboard Relay
// #   Connect the Dust sensor to the xboard relay
// #      LED driver pin - D3
// #      Signal pin     - A0


#include <Metro.h> //Include Metro library 
Metro sensorManager = Metro(500,true); // Manage the sensor data update rate
// Should be longer than 300ms
Metro udpTransManager = Metro(200,true);    //
Metro udpRecevManager = Metro(50,true);    //

#include <SPI.h>
#include <Ethernet.h>
#include <EthernetUdp.h>
#include <aJSON.h>
#define LOCALPORT 8888
#define REMOTEPORT 8889
#define BTN1PORT 5
#define BTN2PORT 5

byte packetBuffer[8]; //buffer to hold incoming packet

EthernetUDP Udp;
byte mac[] = { 0xDE, 0xAD, 0xBE, 0xEF, 0xFE, 0xED };
IPAddress serverIP(192,168,2,214);
IPAddress testIP(255,255,255,255);
IPAddress gateway(192,168,2, 1);
IPAddress subnet(255, 255, 0, 0);

boolean bSwitch1State = false;
boolean bSwitch2State = false;

#define DHT11_PIN 1      // define Digital pin 2 of the Arduino Leonardo (The port number)
#define ledPower 3       // define Digital pin 3 of the Arduino Leonardo (The digital number)
#define measurePin 0       // define Digital pin 3 of the Arduino Leonardo

void setup()
{
	initSensor();
	Serial.begin(9600);
	while (!Serial) {
		; // wait for serial port to connect. Needed for Leonardo only
	}
	Serial.println("Ready");
	initEthernet();
}

int temp = 0;
int humidity = 0;
float dustVal = 0;

void loop()     
{
	if(sensorManager.check()){
		//    unsigned long timer = millis();
		get_DHT_Data();
		dustVal = get_DustSensor_Data();
		//    Serial.print("sensor Thread:");
		//    Serial.println(millis() - timer);
	}
	if(udpTransManager.check()){
		//    unsigned long timer = millis();
		TransmitMessage(humidity,temp,dustVal);
		//    Serial.print("udp trans Thread:");
		//    Serial.println(millis() - timer);
	}

	if(udpRecevManager.check()){
		//    unsigned long timer = millis();
		ReceiveMessage();
		//    Serial.print("udp receiv Thread:");
		//    Serial.println(millis() - timer);
	}
}


/************** communication via ethernet (UDP connection) ********/

void initEthernet(){  
	if (Ethernet.begin(mac) == 0) {
		Serial.println("Failed to configure Ethernet using DHCP");
		// no point in carrying on, so do nothing forevermore:
		for(;;)
			;
	}
	Udp.begin(LOCALPORT);
}

void TransmitMessage(int humidity,int temp,float dust){

	char* ReplyBuffer = getMessage(humidity,temp,dust); 
	// send a reply, to the IP address and port that sent us the packet we received
	//Udp.beginPacket(Udp.remoteIP(), Udp.remotePort());
	Serial.println(ReplyBuffer);

	Udp.beginPacket(testIP, REMOTEPORT);
	//Udp.write(ReplyBuffer,strlen(ReplyBuffer));
	Udp.write(ReplyBuffer);
	Udp.endPacket();
	//Serial.println(ReplyBuffer);
	free(ReplyBuffer);

}

void ReceiveMessage(){

	int packetSize = Udp.parsePacket();
	if(packetSize) {
		//    Serial.print("From ");
		//    IPAddress remote = Udp.remoteIP();
		//    for (int i =0; i < 4; i++)
		//    {
		//      Serial.print(remote[i], DEC);
		//      if (i < 3)
		//      {
		//        Serial.print(".");
		//      }
		//    }
		//    Serial.print(", port ");
		//    Serial.println(Udp.remotePort());

		// read the packet into packetBufffer
		Udp.read(packetBuffer,UDP_TX_PACKET_MAX_SIZE);
		//    Serial.println("Contents:");
		//    Serial.println(packetBuffer[0]);
		//    Serial.println(packetBuffer[1]);

		if(packetBuffer[0] == 1) {
			if(packetBuffer[1] == 1 && !bSwitch1State) {
				analogWrite(BTN1PORT, 255);
				digitalWrite(7,HIGH);
				bSwitch1State = true;
				//       Udp.beginPacket(Udp.remoteIP(), Udp.remotePort());
				//       //Udp.beginPacket(testIP, REMOTEPORT);
				//       //Udp.write(ReplyBuffer,strlen(ReplyBuffer));
				//       Udp.write("abc");
				//       Udp.endPacket();
			}
			else if(packetBuffer[1] == 0 && bSwitch1State) {
				analogWrite(BTN1PORT, 0);
				digitalWrite(7,LOW);
				bSwitch1State = false;
			}
		}
		else if(packetBuffer[0] == 2) {
			if(packetBuffer[1] == 1 && !bSwitch2State) {
				analogWrite(BTN2PORT, 255);
				digitalWrite(8,HIGH);
				bSwitch2State = true;
				//       Udp.beginPacket(Udp.remoteIP(), Udp.remotePort());
				//       //Udp.beginPacket(testIP, REMOTEPORT);
				//       //Udp.write(ReplyBuffer,strlen(ReplyBuffer));
				//       Udp.write("abc");
				//       Udp.endPacket();
			}
			else if(packetBuffer[1] == 0 && bSwitch2State) {
				analogWrite(BTN2PORT, 0);
				digitalWrite(8,LOW);
				bSwitch2State = false;
			}
		}

	}

}

char* getMessage(int humidity,int temp,float dust) {
	aJsonObject* root = aJson.createObject();
	if (root != NULL) {
	} 
	else {
		return NULL;
	}

	if(bSwitch1State) {
		aJson.addNumberToObject(root, "switch1State", 1);
	}
	else {
		aJson.addNumberToObject(root, "switch1State", 0);
	}

	if(bSwitch2State) {
		aJson.addNumberToObject(root, "switch2State", 1);
	}
	else {
		aJson.addNumberToObject(root, "switch2State", 0);
	}

	aJson.addNumberToObject(root, "humidity", humidity);
	aJson.addNumberToObject(root, "temperature", temp);
	aJson.addNumberToObject(root, "dust", dust);

	//  aJson.addItemToObject(root, "name", aJson.createItem(
	//  "DFRobot"));
	//  aJsonObject* fmt = aJson.createObject();
	//  if (fmt != NULL) {
	//    aJson.addItemToObject(root, "value", fmt);
	//    
	//    aJson.addNumberToObject(fmt, "humidity", humidity);
	//    aJson.addNumberToObject(fmt, "temperature", temp);
	//    if(bSwitch1State)
	//    {
	//      aJson.addNumberToObject(fmt, "lightState", 1);
	//    }
	//    else
	//    {
	//      aJson.addNumberToObject(fmt, "lightState", 0);
	//    }


	char* string = aJson.print(root);
	aJson.deleteItem(root);
	return string;
}


/************** measure and report Free ram ***********/
int freeRam(void) {
	extern int  __bss_end; 
	extern int  *__brkval; 
	int free_memory; 
	if((int)__brkval == 0) {
		free_memory = ((int)&free_memory) - ((int)&__bss_end); 
	}
	else {
		free_memory = ((int)&free_memory) - ((int)__brkval); 
	}
	return free_memory; 
} 

/************** Measure temperature and humidity ***********/

void initSensor(){
	DDRD |= _BV(DHT11_PIN);   
	PORTD |= _BV(DHT11_PIN);  

	pinMode(7,OUTPUT);
	pinMode(8,OUTPUT);
	digitalWrite(7,LOW);
	//  digitalWrite(7,HIGH);
	digitalWrite(8,LOW);
	//  digitalWrite(8,HIGH);
	pinMode(ledPower,OUTPUT);
	digitalWrite(ledPower,HIGH);
}

void get_DHT_Data(){  // get data from the DHT11 sensor

	byte dht11_dat[5];    
	byte dht11_in;
	byte i;// start condition

	PORTD &= ~_BV(DHT11_PIN);    // 1. pull-down i/o pin for 18ms
	delay(18);
	PORTD |= _BV(DHT11_PIN);     // 2. pull-up i/o pin for 40us
	delayMicroseconds(1);
	DDRD &= ~_BV(DHT11_PIN);     //let analog port 0 be input port 
	delayMicroseconds(40);      

	dht11_in = PIND & _BV(DHT11_PIN);  // read only the input port 0
	if(dht11_in) {
		Serial.println("dht11 start condition 1 not met"); // wait for DHT response signal: LOW
		delay(1000);
		return;
	}
	delayMicroseconds(80);
	dht11_in = PIND & _BV(DHT11_PIN); //  
	if(!dht11_in) {
		Serial.println("dht11 start condition 2 not met");  //wair for second response signal:HIGH
		return;
	}

	delayMicroseconds(80);// now ready for data reception
	for (i=0; i<5; i++) {  
		dht11_dat[i] = read_dht11_dat();
	}  //recieved 40 bits data. Details are described in datasheet

	DDRD |= _BV(DHT11_PIN);      //let analog port 0 be output port after all the data have been received
	PORTD |= _BV(DHT11_PIN);     //let the  value of this port be '1' after all the data have been received
	byte dht11_check_sum = dht11_dat[0]+dht11_dat[1]+dht11_dat[2]+dht11_dat[3];// check check_sum
	if(dht11_dat[4]!= dht11_check_sum) {
		Serial.println("DHT11 checksum error");
	}
	humidity = dht11_dat[0];
	temp = dht11_dat[2];

	//  Serial.print("Current humdity = ");
	//  Serial.print(humidity, DEC);
	//  Serial.print("%  ");
	//  Serial.print("temperature = ");
	//  Serial.print(temp, DEC);
	//  Serial.println("C  ");
}


byte read_dht11_dat() {
	byte i = 0;
	byte result=0;
	for(i=0; i< 8; i++) {
		while(!(PIND & _BV(DHT11_PIN)))
		{};  // wait  forever until anlog input port 0 is '1'   (NOTICE: PIND reads all the analog input ports 
		//and  _BV(X) is the macro operation which pull up positon 'X'to '1' and the rest positions to '0'. it is equivalent to 1<<X.) 
		delayMicroseconds(30);
		if(PIND & _BV(DHT11_PIN))  //if analog input port 0 is still '1' after 30 us
			result |=(1<<(7-i));     //this position is 1
		while((PIND & _BV(DHT11_PIN)));  // wait '1' finish
	}
	return result;
}

/************** Get dust sensor data **********/

float get_DustSensor_Data(){
	const int samplingTime = 280;
	const int deltaTime = 40;
	const int sleepTime = 9680;
	digitalWrite(ledPower,LOW); // power on the LED
	delayMicroseconds(samplingTime);

	int voMeasured = analogRead(measurePin); // read the dust value

	delayMicroseconds(deltaTime);
	digitalWrite(ledPower,HIGH); // turn the LED off
	delayMicroseconds(sleepTime);

	// 0 - 5V mapped to 0 - 1023 integer values
	// recover voltage
	float calcVoltage = voMeasured * (5.0 / 1024.0);

	// linear eqaution taken from http://www.howmuchsnow.com/arduino/airquality/
	// Chris Nafis (c) 2012
	float dustDensity = 0.17 * calcVoltage - 0.1;

	//  Serial.print("Raw Signal Value (0-1023): ");
	//  Serial.print(voMeasured);

	//  Serial.print(" - Voltage: ");
	//  Serial.print(calcVoltage);

	//  Serial.print(" Dust Density:");
	//  Serial.print(dustDensity);
	//  Serial.println(" mg/m3");

	return dustDensity;

}

